(function (global) {
  "use strict";

  global.FRCC = global.FRCC || {};
  global.FRCC.Portal = global.FRCC.Portal || {};
  global.FRCC.Portal.Accessibility = (function () {
    /**
     * Global set of accessibility modifications to be made on the portal
     *
     * This will be executed from the EYGA Header Web Template so it is run on every page
     */
    var loadGlobalAccessibilityModifications = function (configuration) {
      insertSpanElements(configuration.targetingObject);
      insertHelpSpan(configuration.newWindowHtmlSnippet);
      improveHeaderAccessibility();
      updateNativeTables();
      updateOfflineNotification();
      updateRestrictedPages(configuration?.portalTitle);
      insertDescriptionBelowTable();
      insertAriaDescribedBy();
      helpTextStyle();
      addHelpPopoverForMultiselect();
      makeTablesHorizontallyScrollable();
      addAutoCompleteAttribute();
      applyTablePaddingActionMenuOffset();
    };

    /**
     * @param {object} targetingObject - key/value object containing selectors:insertMethod
     *
     * @example
     * insertSpanElements({
     *  'select': 'after',
     *  'input[type="radio"]': 'after',
     *  '.boolean-radio > label': 'prepend'
     * })
     *
     * This is an accessbility-specific function that was needed due to many fields generating without proper
     *      icon identification on entityforms within the portal.
     */
    var insertSpanElements = function (targetingObject) {
      // Cache default jQuery selector
      var $ = jQuery;

      // Native subgrids are generated within iframe elements on entity forms
      // Attaching a handler that will call the function to insert span elements when the iframe has completed its loading
      $("iframe[data-page]").on("load", function (e) {
        try {
          _insertSpanElementsHelper(
            targetingObject,
            e.currentTarget.contentWindow.$
          );
        } catch (error) {
          handleError(error);
        }
      });

      _insertSpanElementsHelper(targetingObject, $);
    };

    var _insertSpanElementsHelper = function (targetingObject, $) {
      if (typeof $ === "undefined" || $ === null) {
        return;
      }

      const spanElementHtml = '<span class="ui"></span>';

      for (const property in targetingObject) {
        var selector = property;
        var insertMethod = targetingObject[property];
        switch (insertMethod) {
          case "after":
            $(selector).after(spanElementHtml);
            break;
          case "prepend":
            $(selector).prepend(spanElementHtml);
            break;
          default:
            break;
        }
      }
    };

    var resetPageTabbing = function () {
      $("body").focus();
      return;
    };

    /**
     * Add a popover element for the Microsoft Multiselect Option List component - MSOS SOL
     * Adding a popover so when the selected items <ul> is focused this popover will be visible with keyboard usage instructions
     * This control could be in the document or in a iFrame so providing a mutation observer to help find the elements in each
     */
    var addHelpPopoverForMultiselectHelper = function ($, window) {
      if (typeof $ === "undefined" || $ === null) {
        return;
      }

      var _document = window;
      var observer = new MutationObserver(function () {
        if ($(_document).find(".msos-selecteditems").length) {
          var multiselectSelectedOptions = $(".msos-selecteditems");
          var popoverMessage =
            "Use the arrow keys to navigate through the list of currently selected items. Press delete or backspace to deselect an item.";

          var popover = `<div class="popover top eyga-msos-popover" aria-hidden="true"><div class="arrow"></div><div class="popover-content">${popoverMessage}</div></div>`;

          if (multiselectSelectedOptions.length) {
            multiselectSelectedOptions.after(popover);
          }
          observer.disconnect();
        }
      });
      observer.observe(_document, {
        childList: true,
        subtree: true
      });
    };

    var addHelpPopoverForMultiselect = function () {
      // Cache default jQuery selector
      var $ = jQuery;

      $("iframe[data-page]").on("load", function (e) {
        try {
          addHelpPopoverForMultiselectHelper(
            e.currentTarget.contentWindow.$,
            e.currentTarget.contentWindow.document
          );
        } catch (error) {
          handleError(error);
        }
      });

      addHelpPopoverForMultiselectHelper($, document);
    };

    var insertDescriptionBelowTable = function () {
      $(".description.below").find("table").attr("role", "presentation");

      // Native subgrids are generated within iframe elements on entity forms
      // Attaching a handler that will call the function to insert span elements when the iframe has completed its loading
      $("iframe[data-page]").on("load", function (e) {
        try {
          var $ = e.currentTarget.contentWindow.$;
          if ($) {
            $("#EntityFormControl_EntityFormView")
              .find(".description.below")
              .find("table")
              .attr("role", "presentation");
          }
        } catch (error) {
          handleError(error);
        }
      });
    };

    /**
     * This function adds an aria-describedby attribute to input elements that is set to the associated
     * label's id
     */
    var insertAriaDescribedBy = function () {
      var $ = jQuery;
      $("iframe[data-page]").on("load", function (e) {
        try {
          setTimeout(function () {
            insertAriaDescribedByHelper(e.currentTarget.contentWindow.$);
          }, 2000);
        } catch (error) {
          handleError(error);
        }
      });
      setTimeout(function () {
        insertAriaDescribedByHelper($);
      }, 2000);
    };
    /**
     * Helper function that holds logic
     */
    var insertAriaDescribedByHelper = function ($) {
      if (!$) {
        return;
      }

      $("input").each(function () {
        //check here if there is a radio-cell in the parent of info req
        var ifHidden = $(this).attr("type");
        var controlClass = $(this).closest("div.control");
        var isHelpText = controlClass.next().hasClass("description below");
        var infoClass = controlClass.prev();
        let associatedLabelId;
        if (ifHidden !== "hidden" && isHelpText) {
          if (infoClass.parent().hasClass("boolean-radio-cell")) {
            associatedLabelId = infoClass.children(":first").attr("id");
            infoClass
              .parent()
              .attr("aria-describedby", associatedLabelId + "_help");
            controlClass.next().attr("id", associatedLabelId + "_help");
          } else {
            associatedLabelId = infoClass.children(":first").attr("id");
            $(this).attr("aria-describedby", associatedLabelId + "_help");
            controlClass.next().attr("id", associatedLabelId + "_help");
          }
        }
      });

      $("textarea").each(function () {
        var ifHidden = $(this).attr("type");
        var controlClass = $(this).closest("div.control");
        var infoClass = controlClass.prev();
        var isHelpText = controlClass.next().hasClass("description below");
        if (ifHidden !== "hidden" && isHelpText) {
          var associatedLabelId = infoClass.children(":first").attr("id");
          $(this).attr("aria-describedby", associatedLabelId + "_help");
          controlClass.next().attr("id", associatedLabelId + "_help");
        }
      });

      $("select").each(function () {
        var ifHidden = $(this).attr("type");
        var controlClass = $(this).closest("div.control");
        var infoClass = controlClass.prev();
        var isHelpText = controlClass.next().hasClass("description below");
        if (ifHidden !== "hidden" && isHelpText) {
          var associatedLabelId = infoClass.children(":first").attr("id");
          $(this).attr("aria-describedby", associatedLabelId + "_help");
          controlClass.next().attr("id", associatedLabelId + "_help");
        }
      });
    };

    /**
     * Removes in line styling that interfered with help text readability
     */
    var helpTextStyle = function () {
      var $ = jQuery;
      $("iframe[data-page]").on("load", function (e) {
        try {
          helpTextStyleHelper(e.currentTarget.contentWindow.$);
        } catch (error) {
          handleError(error);
        }
      });
      helpTextStyleHelper($);
    };

    var helpTextStyleHelper = function ($) {
      if (!$) {
        return;
      }

      $(".description").each(function () {
        $(this).children(":first").removeAttr("style");
        $(this).children().removeAttr("style");
      });
      $("div.description span").removeAttr("style");
    };

    /**
     * This function improves keyboard accessibility for the main navigation header.
     *
     * If the Escape key is pressed when the focus is within he header and a submenu is open, it removes the
     *      "in" class from the dropdown menu to close it.
     *
     * If the Up or Down keys are pressed and the focus is within the context of the navbar-links menu, it will
     *      mimic tab and shift-tab functionality
     */
    var improveHeaderAccessibility = function () {
      var $nav = $('nav[aria-label="main"]');
      var $navbarToggle = $(".navbar-toggle");
      var $webLinks = $(".weblinks");
      var $weblinkDropdown = $(".weblink.dropdown");

      document.addEventListener("keydown", function (e) {
        if (
          e.code === "Escape" &&
          $nav.has(e.target).length &&
          $nav.find('[aria-expanded="true"]').length
        ) {
          $("#navbar").removeClass("in");
        }

        if (
          (e.code === "ArrowDown" || e.code === "ArrowUp") &&
          ($navbarToggle[0] === e.target || $webLinks.has(e.target).length) &&
          !$weblinkDropdown.has(e.target).length &&
          $nav.find('[aria-expanded="true"]').length
        ) {
          var currentElement = $(document.activeElement);
          var nextElement = $(":tabbable").eq(
            $(":tabbable").index(currentElement) +
              (e.code === "ArrowDown" ? 1 : -1)
          );
          if (!nextElement.hasClass("navbar-links") && e.code === "ArrowDown") {
            e.preventDefault();
            return;
          }
          nextElement.focus();
          e.preventDefault();
        }
      });
    };

    /**
     * This function updates the classes used for native tables loaded onto the portal
     *
     * A MutationObserver is required because the native tables are loaded dynamically after
     *      the page/document is rendered
     *
     * table-fluid causes screen readers to not be able to find the table properly and needs to be replaced
     *
     * This function will replace the table-fluid class with table-mobile: a custom class that copies
     *      the styles used in table-fluid
     */
    var updateNativeTables = function () {
      // Native subgrids are generated within iframe elements on entity forms
      // Attaching a handler that will call the function to create a mutation observer when the iframe has finished loading
      $("iframe[data-page]").on("load", function (e) {
        try {
          _registerMutationObserver(
            e.currentTarget.contentWindow.document,
            null,
            false,
            { childList: true, subtree: true },
            _updateNativeTablesHelper
          );
        } catch (error) {
          handleError(error);
        }
      });

      _registerMutationObserver(
        document,
        "main",
        false,
        { childList: true, subtree: true },
        _updateNativeTablesHelper
      );

      _registerMutationObserver(
        document,
        ".view-pagination:not(.core-pagination-wrapper)",
        false,
        { childList: true, subtree: true },
        _updateNativePaginationHelper
      );
    };

    /**
     * Wrap table elements with another div to allow horizontal scrolling
     */
    var makeTablesHorizontallyScrollable = function () {
      $("iframe[data-page]").on("load", function (e) {
        try {
          makeTablesHorizontallyScrollableHelper(
            e.currentTarget.contentWindow.document
          );
        } catch (error) {
          handleError(error);
        }
      });

      makeTablesHorizontallyScrollableHelper(document);
    };

    var makeTablesHorizontallyScrollableHelper = function (documentContext) {
      const tables = documentContext.querySelectorAll(".table");
      const wrapperClass = "table-scroll-wrapper";

      tables.forEach((table) => {
        // Skip adding scroll wrapper if there is one already
        if (table.closest(`.${wrapperClass}`)) {
          return;
        }

        var scrollWrapper = document.createElement("div");
        scrollWrapper.classList.add(wrapperClass);
        table.parentNode.insertBefore(scrollWrapper, table);
        scrollWrapper.appendChild(table);
      });
    };

    var _updateNativePaginationHelper = function (mutationsList) {
      mutationsList.forEach((mutation) => {
        if (
          mutation.target.classList.contains("view-pagination") &&
          mutation.addedNodes.length > 0
        ) {
          var nativePaginationWrapper = $(
            ".view-pagination:not(.core-pagination-wrapper)"
          );

          nativePaginationWrapper
            .find("ul.pagination")
            .wrap("<nav aria-label='pagination'></nav>");

          nativePaginationWrapper.find("a").each(function () {
            $(this).removeAttr("role");

            var ariaLabel = $(this).attr("aria-label");

            if (ariaLabel == "Previous page" || ariaLabel == "Next page") {
              var newTitle = $(this).attr("aria-label");
              $(this).attr("title", newTitle);
              $(this).removeAttr("data-toggle");
              $(this).contents().wrap("<span aria-hidden='true'></span>");
            } else if ($(this).attr("data-page") == "..") {
              $(this).contents().wrap("<span aria-hidden='true'></span>");
              $(this).attr(
                "aria-label",
                "ellipsis indicating non-visible pages"
              );
            } else if (!$(this).parent().hasClass("active")) {
              //if it's a number but not active
              $(this).attr("aria-current", "false");
            } else {
              //number that is active
              var pageNum = $(this).attr("data-page");
              $(this).attr("aria-label", `page ${pageNum}`);
            }

            if ($(this).parent().hasClass("disabled")) {
              $(this).removeAttr("href");
              $(this).attr("aria-disabled", "true");
            } else {
              $(this).attr("href", "javascript:void(0)");
            }
          });
        }
      });
    };

    var _updateNativeTablesHelper = function (mutationsList) {
      mutationsList.forEach((mutation) => {
        mutation.addedNodes.forEach((node) => {
          if (node.nodeName.toLowerCase() === "table") {
            node.removeAttribute("aria-relevant"); // Remove aria-relevant="additions" - not supported by most browsers

            const classNames = node.classList;
            if (
              classNames.contains("table-hover") &&
              !classNames.contains("table-custom")
            ) {
              node.querySelector("th").style.width = "5%";
            }

            classNames.forEach((e) => {
              if (!e.contains("table-fluid")) {
                return;
              }

              var n = e.replace("fluid", "mobile");
              classNames.replace(e, n);
            });
          } else if (node.nodeName.toLowerCase() === "tbody") {
            const parentNode = node.parentNode;
            if (parentNode.getAttribute("role") !== "grid") {
              return;
            }

            const tdNodes = node.querySelectorAll("td");
            tdNodes.forEach((td) => {
              td.classList.add("text-wrap");
            });
          }
        });
      });
    };

    /**
     * Set of accessibility modifications to be made on the portal against the native offlineNotificationBar
     */
    var updateOfflineNotification = function () {
      $("#offlineNotificationBar").attr("role", "alert");
      $("#web").attr("aria-hidden", "true");
      $("#close").attr({
        role: "button",
        tabindex: "0",
        style: "top:0.8rem"
      });
      $("#close").on("keypress", function (e) {
        if (e.key !== "Enter") {
          return;
        }

        $(this).click();
      });
    };

    /**
     * Set of accessibility modifications to be made on the portal's "Restricted Pages"
     *
     * Restricted Pages are web pages where the HTML cannot be edited via a Web Template such as SignIn, Register, Redeem Invitation
     */
    var updateRestrictedPages = function (portalTitle) {
      const webpageTitle = window?.location?.pathname;
      if (!webpageTitle) {
        return;
      }

      const restrictedPages = [
        "/signin",
        "/register",
        "/account/login/register"
      ];
      if (!restrictedPages.includes(webpageTitle.toLowerCase())) {
        return;
      }

      _updatePageTitles(portalTitle);
      _updatePageStructure(webpageTitle);
      _updateRestrictedPageAccessibility();
    };

    /**
     * Updates the restricted page's document.title attribute to be consistent with formatting across the Portal
     */
    var _updatePageTitles = function (portalTitle) {
      if (!portalTitle) {
        return;
      }

      _registerMutationObserver(
        document,
        "title",
        false,
        { childList: true, subtree: true },
        (mutationsList) => {
          _updatePageTitle(mutationsList, portalTitle);
        }
      );
    };

    var _updatePageTitle = function (mutationsList, portalTitle) {
      const documentTitle = document.title;
      const parts = documentTitle.split("-");
      const pageTitle = parts.slice(-1);
      if (pageTitle.length === 0) {
        return;
      }

      if (documentTitle === `${portalTitle} - ${pageTitle[0].trim()}`) {
        return;
      }

      document.title = `${portalTitle} - ${pageTitle[0]}`;
    };

    /**
     * Updates the restricted page's HTML structure to be consistent with structures used on other Web Pages throughout
     *      the portal
     */
    var _updatePageStructure = function (webpageTitle) {
      const pageTitle = webpageTitle.replace(/\//g, "").toLowerCase().trim();

      $("#content").addClass("body-content");
      $("#content").unwrap();
      $("#content").wrap(
        `<main id="content-container" class="page-content page-${pageTitle}" role="main"></main>`
      );
      $("#mainContent").removeClass("page-content");
      $("#mainContent").addClass("container");
      $(".nav.nav-tabs.nav-account").prependTo("#mainContent");
    };

    /**
     * Updates the restricted page's accessibility by removing redundant attributes and modifying validation alerts
     */
    var _updateRestrictedPageAccessibility = function () {
      _registerMutationObserver(
        document,
        ".form-horizontal",
        true,
        { childList: true, subtree: true },
        _modifyRestrictPageValidationAlerts
      );

      $(".nav.nav-tabs.nav-account > li > a").attr("tabindex", "0");
      $(".form-horizontal div.alert-danger").attr({
        role: "alert",
        "aria-live": "assertive"
      });
      $("*[title]").removeAttr("aria-label");
      $("*[title]").removeAttr("title");
    };

    var _modifyRestrictPageValidationAlerts = function () {
      $(".form-horizontal div.alert-danger li:visible a").each(
        function (index, ele) {
          var $ele = $(ele);
          var id = $ele.attr("href");
          $(id).closest(".form-group").addClass("has-error");
        }
      );
    };

    var _registerMutationObserver = function (
      _document,
      querySelector,
      multiple,
      config,
      handler
    ) {
      const observer = new MutationObserver(handler);

      const targetNodes = multiple
        ? querySelector
          ? _document.querySelectorAll(querySelector)
          : [_document.body]
        : querySelector
          ? _document.querySelector(querySelector) !== null
            ? [_document.querySelector(querySelector)]
            : [_document.body]
          : [_document.body];

      targetNodes.forEach((node) => observer.observe(node, config));
    };

    var _registerResizeObserver = function (
      _document,
      querySelector,
      multiple,
      handler
    ) {
      const observer = new ResizeObserver(handler);

      const targetNodes = multiple
        ? querySelector
          ? _document.querySelectorAll(querySelector)
          : [_document.body]
        : querySelector
          ? _document.querySelector(querySelector) !== null
            ? [_document.querySelector(querySelector)]
            : [_document.body]
          : [_document.body];

      targetNodes.forEach((node) => observer.observe(node));
    };

    var addAutoCompleteAttribute = function () {
      var $ = jQuery;
      $("iframe[data-page]").on("load", function (e) {
        try {
          addAutoCompleteAttributeHelper(e.currentTarget.contentWindow.$);
        } catch (error) {
          handleError(error);
        }
      });
      addAutoCompleteAttributeHelper($);
    };

    /**
     * This adds required autocomplete attribute with correct token to input elements on an application
     */
    var addAutoCompleteAttributeHelper = function ($) {
      if (typeof $ === "undefined" || $ === null) {
        return;
      }

      var allInputs = $("input, select");
      allInputs.each(function () {
        const inputId = $(this).attr("id");
        if (
          inputId?.includes("address") &&
          inputId?.includes("street") &&
          inputId?.includes("1")
        ) {
          $(this).attr("autocomplete", "address-line1");
        } else if (
          inputId?.includes("address") &&
          inputId?.includes("street") &&
          inputId?.includes("2")
        ) {
          $(this).attr("autocomplete", "address-line2");
        } else if (inputId?.indexOf("city") !== -1) {
          $(this).attr("autocomplete", "address-level2");
        } else if (inputId?.indexOf("state") !== -1) {
          $(this).attr("autocomplete", "address-level1");
        } else if (
          inputId?.indexOf("postalcode") !== -1 ||
          inputId.indexOf("zip") !== -1
        ) {
          $(this).attr("autocomplete", "postal-code");
        } else if (inputId?.indexOf("phone") !== -1) {
          $(this).attr("autocomplete", "tel");
        } else if (inputId?.indexOf("email") !== -1) {
          $(this).attr("autocomplete", "email");
        }
      });
    };

    /**
     * Apply a padding and negative margin to table scroller so table doesnt clip action menu
     * CSS will pull in document CSS var to use as the value on non-mobile layouts
     */
    var applyTablePaddingActionMenuOffset = function () {
      $("table").on("show.bs.dropdown", function ({ target }) {
        /* If table is part of the application, dont do this
         * Application tables use fixed position to position dropdown
         */
        if ($("table").closest(".view-grid").length) {
          return;
        }

        const dropdownMenu = target.querySelector(".dropdown-menu");
        const dropdownMenuHeight = $(dropdownMenu).outerHeight();
        const heightAllowance = 5; // Value to add to the height so its has some wiggle room

        document.documentElement.style.setProperty(
          "--dropdownMenuHeight",
          `${dropdownMenuHeight + heightAllowance}px`
        );
      });
    };

    /**
     * Sets the target attribute to "_blank" for links in description elements.
     *
     * This function sets the target attribute to "_blank" for links in description elements within the main page content
     * and appends a provided HTML snippet for additional accessibility or information.
     *
     * @param {string} newWindowHtmlSnippet - The HTML snippet to be appended to each link.
     */
    const insertHelpSpan = function (newWindowHtmlSnippet) {
      $("main.page-content.page-application a")
        .attr("target", "_blank")
        .append(newWindowHtmlSnippet);
    };

    /**
     * Application-specific set of accessibility modifications to be made on the portal
     *
     * This will be executed from the EYGA Templates: Application Create/Edit/View Web Templates
     */
    var loadApplicationAccessibilityModifications = function () {
      _removeRedundantAttributes();
      _modifyValidationAlerts();
      _modifyTableAlerts();
      _updateTableResponsive();
      _fixTableActionsHeader();
      _addPlaceholderText();
      _modifyProgressIndicatorModal();
      _removeRoleDescriptions();
      _modifyRequiredFieldLabels();
      _addApplicationLoadingIndicators();
      _modifyNativeModalIFrames();
      _hideInstructionsIfEmpty();
      _reverseRadioBooleanCells();
      /**
       * Requires await when using web session
       * await _modifyMultiStepNavigationElements();
       * also make loadApplicationAccessibilityModifications async
       */
      _modifyMultiStepNavigationElements();
      _uncheckDefaultedRadioButtons();
      _removeFormElementDisabledAttributes();
      _removeAriaLabels();
      _removeAriaReadOnlyOnTables();
      _applyDatePickerAriaAttributes();
      _addTableCaptions();
      _addDocumentDownloadAccessibleText();
      _updateTableActionIcons();
      _modifyMultilineTextRowHeight();
    };

    /**
     * Removes "aria-label" and "title" attributes from required fields on entityforms within the portal.
     *
     * This is an accessbility-specific function that was needed due to required fields generating
     *      with additional attributes that non-required fields were not generating with.
     * The aria-label and title attributes were causing screen readers to be read back the same information
     *      to the user, causing confusing and redundancy.
     */
    var _removeRedundantAttributes = function () {
      var $ = jQuery;
      $("iframe[data-page]").on("load", function (e) {
        try {
          _removeRedundantAttributesHelper(e.currentTarget.contentWindow.$);
        } catch (error) {
          handleError(error);
        }
      });
      _removeRedundantAttributesHelper($);
    };

    var _removeRedundantAttributesHelper = function ($) {
      if (typeof $ === "undefined" || $ === null) {
        return;
      }

      $('.required + .control > [aria-required="true"]').removeAttr(
        "aria-label title"
      );
      $(".required + .control > select").removeAttr("title");
      $('input[type="text"][required]')
        .removeAttr("required")
        .attr("aria-required", "true");
      $("input[readonly]").removeAttr("aria-label");
      $("input[readonly]").removeAttr("aria-label");

      $(
        "input[aria-roledescription], select[aria-roledescription], textarea[aria-roledescription]"
      ).removeAttr("aria-roledescription");
    };

    /**
     * Adds role="alert" to the validation summary elements on entityforms within the portal.
     *
     * This is an accessibility-specific function that was needed due to the validation areas on the form
     *      not having the proper role to inform screen reader users of the intended function of the element(s)
     *      on their screen.
     */
    var _modifyValidationAlerts = function () {
      var $ = jQuery;

      $("iframe[data-page]").on("load", function (e) {
        try {
          _modifyValidationAlertsHelper(
            e.currentTarget.contentWindow.document,
            e.currentTarget.contentWindow.$
          );
        } catch (error) {
          handleError(error);
        }
      });

      _modifyValidationAlertsHelper(document, $);
    };

    var _modifyValidationAlertsHelper = function (documentContext, $) {
      if (!$) {
        return;
      }

      var validationBlockSelector =
        ".validation-summary.alert.alert-error.alert-danger.alert-block";
      var validationBlock = $(validationBlockSelector);
      validationBlock.attr("role", "alert");

      _registerMutationObserver(
        documentContext,
        validationBlockSelector,
        true,
        { childList: true, subtree: true },
        _addErrorClasses
      );

      $('input[value="x"]').val("×");
    };

    var _modifyMultilineTextRowHeight = function () {
      var $ = jQuery;
      $("iframe[data-page],iframe[data-formname]").on("load", function (e) {
        try {
          _modifyMultilineTextRowHeightHelper(e.currentTarget.contentWindow.$);
          _resizeQuickviewFormHeight();
        } catch (error) {
          handleError(error);
        }
      });
      _modifyMultilineTextRowHeightHelper($);
    };

    var _modifyMultilineTextRowHeightHelper = function ($) {
      if (!$) {
        return;
      }

      $("textarea.textarea,textarea.form-control").attr("rows", 10);
    };

    const _resizeQuickviewFormHeight = function () {
      $("iframe[data-formname]").height(
        $("iframe[data-formname]")
          .contents()
          .find(".container.quickform")
          .height()
      );
    };

    var _addErrorClasses = function (mutationsList) {
      mutationsList.forEach((mutation) => {
        const $parentForm = $(mutation.target).closest(".entity-form");

        $(mutation.target)
          .closest(".entity-form")
          .find(".has-error")
          .removeClass("has-error");
        mutation.target.setAttribute("role", "alert");
        mutation.target.setAttribute("aria-live", "assertive");

        mutation.addedNodes.forEach((node) => {
          if (node.nodeName.toLowerCase() !== "ul") {
            return;
          }

          node.removeAttribute("role");
          node.childNodes.forEach((childNode) => {
            var href = childNode.childNodes[0]
              .getAttribute("href")
              .replace("#", "");
            if (href.includes("_label")) {
              $parentForm
                .find(`#${href}`)
                .parent()
                .siblings(".control")
                .addClass("has-error");
            } else if (href.includes("WebResource_FileUpload_")) {
              document
                .getElementById(href)
                .parentElement.previousElementSibling.classList.add(
                  "has-error"
                );
            } else {
              $(`#${href}`).parent().addClass("has-error");
            }
          });
        });
      });
    };

    var _modifyTableAlerts = function () {
      $(".view-empty").removeAttr("role");
      $(".view-access-denied").attr("role", "alert");
      $(".view-error").attr("role", "alert");
    };

    /**
     * Adds placeholder text to select controls on entityforms within the portal
     *
     * This is an accessibility-specific function that was needed due to selects natively showing no placeholder
     *      text when no option was selected
     */
    var _addPlaceholderText = function () {
      $(".form-control.picklist option:first-child").text(
        "Please select an option"
      );
    };

    /**
     * Adds aria-atomic="true" and aria-live="assertive" to parent-level element of the progressIndicator element
     *
     * Since the Progress Indicator is loaded via a Content Snippet, it comes with extra baggage - 2 parent divs
     *      are loaded along with the HTML content of the Content Snippet. In order to correctly mark the element
     *      to screen readers as a "live" area, we have to do it at the parent-level
     */
    var _modifyProgressIndicatorModal = function () {
      var progressIndicator = $("#progressIndicator");
      if (progressIndicator.length === 0) {
        return;
      }

      progressIndicator
        .parent()
        .parent()
        .attr({ "aria-atomic": "true", "aria-live": "assertive" });
    };

    /**
     * Removes "aria-roledescription" from all inputs on entityforms within the portal.
     *
     * This is an accessbility-specific function that was needed due the aria-roledescription attribute
     *      overriding the selected attribute and reading the content of aria-roledescription to screen readers.
     */
    var _removeRoleDescriptions = function () {
      $("[aria-roledescription]").removeAttr("aria-roledescription");
    };

    /**
     * Adds <span class="sr-only">*</span> to all required input labels within the portal.
     *
     * This is an accessbility-specific function that was needed due to labels for required inputs
     *      not having proper acessibility for screen-readers.
     */
    var _modifyRequiredFieldLabels = function () {
      $(".info.required > label").each(function () {
        let labelHtml = $(this).html();
        $(this).html(`${labelHtml} <span class="sr-only">*</span>`);
      });
    };

    var _addApplicationLoadingIndicators = function () {
      const submitButtonSelector =
        ".crmEntityFormView .btn.btn-primary.submit-btn";

      const observer = new MutationObserver(_displayLoadingIndicator);
      const config = { attributes: true };
      const targetNodes = document.querySelectorAll(submitButtonSelector);

      targetNodes.forEach((node) => observer.observe(node, config));
    };

    var _displayLoadingIndicator = function (mutationsList) {
      mutationsList.forEach((mutation) => {
        const disabled = mutation.target.disabled;
        if (
          disabled &&
          !mutation.target.classList.contains("read-only-enabled")
        ) {
          FRCC?.Portal?.Utilities?.showProgressIndicator("Saving...");
          return;
        }

        FRCC?.Portal?.Utilities?.hideProgressIndicator();
      });
    };

    var _modifyNativeModalIFrames = function () {
      $("iframe[data-page]").on("load", function (e) {
        const iFrame = $(e.target);
        const iFrameDocument = iFrame[0]?.contentWindow?.document;
        const iFrame$ = e?.currentTarget?.contentWindow?.$;
        if (!iFrame || iFrame.length === 0 || !iFrameDocument || !iFrame$) {
          return;
        }

        _updateModalIFrameHeight(iFrame, iFrame$);
        _registerResizeObserver(iFrameDocument, "td.cell", true, () =>
          _updateModalIFrameHeight(iFrame, iFrame$)
        );
        _registerResizeObserver(
          iFrameDocument,
          "#ValidationSummaryEntityFormControl_EntityFormView li",
          true,
          () => _updateModalIFrameHeight(iFrame, iFrame$)
        );
      });
    };

    var _updateModalIFrameHeight = function (iFrame, iFrame$) {
      const scrollHeight = iFrame$("body").prop("scrollHeight");
      if (scrollHeight === 0) {
        return;
      }

      const visibleHeight = $(iFrame$("#EntityFormPanel:visible"))
        .toArray()
        .reduce((accumulator, ele) => accumulator + $(ele).outerHeight(), 0);
      iFrame.height(visibleHeight);
    };

    var handleError = function (errorMessage, displayError = false) {
      if (displayError) {
        alert(errorMessage);
      }

      console.error(errorMessage);
    };

    /**
     * Parent function that contains a set of functions aimed at adjusting the
     * multistep (advanced form) navigation elements and functionality.
     *
     * @function _modifyMultiStepNavigationElements
     */
    const _modifyMultiStepNavigationElements = function () {
      // Note: Ordering of the functions is important
      _modifyMultiStepListLayout();
      _replaceMultistepFormNavigationHtml();
      /**
       * Requires async on this function _modifyMultiStepNavigationElements
       * await _modifyMultistepFormNavigation();
       */
      _modifyMultiStepListGroupProgressSteps();
    };

    /**
     * Adjusts the markup to handle the two column layout for Multi-Step Form Layouts
     */
    var _modifyMultiStepListLayout = function () {
      var multiStepList = document.querySelector("ol.progress.list-group");

      if (multiStepList) {
        var $multiStepLayoutContainer = $(multiStepList).parent().parent();
        var $multiStepFormContent = $("#WebFormPanel");

        $multiStepLayoutContainer.addClass("multi-step-form-layout");
        $multiStepFormContent.wrap(
          '<div class="multi-step-form-main-content"></div>'
        );
        $(multiStepList)
          .closest("div")
          .wrap(
            '<nav class="multi-step-form-list-group" aria-label="application steps"></nav>'
          );
        $(multiStepList).removeClass("top");
      }
    };

    /**
     * Adjusts the multistep form list group progress to include screen reader text
     */
    var _modifyMultiStepListGroupProgressSteps = function () {
      var multiStepList = document.querySelector("div.progress.list-group");

      if (multiStepList) {
        var listItems = multiStepList.querySelectorAll("a");

        // Hide extra markup not needed for screen readers
        var $multiStepProgressDescription = $(multiStepList).next(".sr-only");
        if ($multiStepProgressDescription.length) {
          $multiStepProgressDescription.remove();
        }

        // Add screen reader text to each list item
        listItems.forEach(function (item) {
          var textContent = "";
          var listItemSRtext = document.createElement("span");
          listItemSRtext.classList.add("sr-only");

          if (item.classList.contains("list-group-item-success")) {
            item.querySelector(".glyphicon").removeAttribute("aria-label");
            item.querySelector(".glyphicon").removeAttribute("role");
            textContent = "Completed";
          } else if (item.classList.contains("active")) {
            textContent = "Current";
          } else if (item.classList.contains("incomplete")) {
            textContent = "Not Completed";
          }

          listItemSRtext.textContent = textContent;
          item.appendChild(listItemSRtext);
        });
      }
    };

    /**
     * Utility function to help make disabled elements more accessible with readonly attributes
     * and disabling unselected values so values cant be changed
     */
    var updateDisabledElements = function (item) {
      item.setAttribute("readonly", "");

      item.removeAttribute("disabled");
      item.removeAttribute("aria-disabled");
    };

    /**
     * Disabled radio buttons cant be read by screen readers or recieve focus,
     * we will add readonly attributes to the :checked:disabled options and remove the disabled attribute
     * Just changing radio buttons to readonly doesnt prevent the value from changing
     */
    var _removeRadioButtonDisabledAttribute = function () {
      var disabledCheckedRadios = document.querySelectorAll(
        'input[type="radio"]:checked:disabled'
      );

      disabledCheckedRadios.forEach((item) => {
        updateDisabledElements(item);
      });
    };

    /**
     * Remove disabled attributes from input elements that are not radios or checkboxes
     */
    var _removeInputDisabledAttribute = function () {
      var disabledInputs = document.querySelectorAll(
        'input:not([type="radio"]):not([type="checkbox"]):disabled'
      );

      disabledInputs.forEach((item) => {
        updateDisabledElements(item);
      });
    };

    /**
     * Remove disabled attributes from select elements, apply disabled to options so value cant be changed
     */
    var _removeSelectDisabledAttribute = function () {
      var disabledSelect = document.querySelectorAll("select:disabled");

      disabledSelect.forEach((item) => {
        updateDisabledElements(item);

        var unselectedOptions = item.querySelectorAll("option:not(:checked)");
        unselectedOptions.forEach((item) => {
          item.setAttribute("disabled", "");
        });
      });
    };

    /**
     * Remove disabled attributes from textarea elements
     */
    var _removeTextareaDisabledAttribute = function () {
      var disabledTextareas = document.querySelectorAll("textarea:disabled");

      disabledTextareas.forEach((item) => {
        updateDisabledElements(item);
      });
    };

    /**
     * Function to call all the form related functions that removed disabled attributes and adds readonly, disables non selection options if necessary so values cannot be changed
     */
    var _removeFormElementDisabledAttributes = function () {
      _removeRadioButtonDisabledAttribute();
      _removeInputDisabledAttribute();
      _removeSelectDisabledAttribute();
      _removeTextareaDisabledAttribute();
    };

    const _replaceMultistepFormNavigationHtml = function () {
      const $ol = $(".multi-step-form-list-group ol");
      const $newDiv = $("<div>", {
        class: $ol.attr("class"),
        id: $ol.attr("id")
      });

      $ol.find("li").each(function () {
        const $li = $(this);
        const $a = $("<a>", {
          class: $li.attr("class"),
          html: $li.html()
        });

        $newDiv.append($a);
      });

      $ol.replaceWith($newDiv);
    };

    /**
     * Commented out until Multistep forms and GetWebFormSessions service is in place
     * 
    const _modifyMultistepFormNavigation = async function () {
      const applicationId = $("#hdn-parentrecordid").val();
      if (!applicationId) {
        $(".multi-step-form-list-group div a:not(.active)").addClass(
          "disabled"
        );
        return;
      }

      try {
        const webFormSessionResponse = await FRCC.Portal.Utilities.asyncAppAjax(
          {
            type: "GET",
            url: `/Services/GetWebFormSessions/?applicationId=${applicationId}`,
            contentType: "application/json"
          }
        );

        const webFormSession = JSON.parse(webFormSessionResponse);
        if (!webFormSession || !webFormSession.Id) {
          return;
        }

        const webFormSteps = webFormSession.WebFormSteps;
        const orderedWebFormSteps = _orderWebFormSteps(webFormSteps);

        _attachMultistepFormNavigationHandlers(
          webFormSession.StepHistory,
          orderedWebFormSteps
        );
      } catch (error) {
        console.error(error);
      }
    };

    const _attachMultistepFormNavigationHandlers = function (
      webFormSessionStepHistory,
      orderedWebFormSteps
    ) {
      const visitedIds = new Set(
        webFormSessionStepHistory.map((step) => step.ID)
      );
      const updatedWebFormSteps = orderedWebFormSteps.map((step) => {
        return {
          ...step,
          Visited: visitedIds.has(step.Id)
        };
      });

      $(".multi-step-form-list-group div a").each(function (index) {
        const $webFormStepLink = $(this);

        const matchedWebFormStep = updatedWebFormSteps.filter(
          (s) => s.Order === index
        )[0];
        $webFormStepLink.attr("data-visited", matchedWebFormStep.Visited);

        const addCheckIcon = () =>
          $webFormStepLink.prepend(
            `<span class="glyphicon glyphicon-ok"></span>`
          );

        if ($webFormStepLink.hasClass("active")) {
          const isReadonly = document.getElementById(
            "application-readonly-alert"
          );

          // Show check on step if app readonly
          if (matchedWebFormStep.Visited && isReadonly) {
            addCheckIcon();
          }

          return;
        }

        if (!matchedWebFormStep || !matchedWebFormStep.Visited) {
          $webFormStepLink.addClass("disabled");
          return;
        }

        $webFormStepLink.attr("tabindex", "0");
        $webFormStepLink.attr("href", "javascript:void(0)");

        if (!$webFormStepLink.hasClass("list-group-item-success")) {
          $webFormStepLink
            .addClass("list-group-item-success")
            .removeClass("incomplete");
          addCheckIcon();
        }

        $(this).click(function (e) {
          _updateWebFormSession(e, matchedWebFormStep);
        });
      });

      // For Download View forms, we want to want to tag which sections have been
       // visited, so we know which Yes / No fields to clear.
      var prevEntityPanel = null;
      $(".accordion-container #EntityFormPanel").each(function (index) {
        const matchedWebFormStep = updatedWebFormSteps.filter(
          (s) => s.Order === index
        )[0];

        $(this).attr("data-completed", false);
        $(this).attr("data-visited", matchedWebFormStep.Visited);
        if (prevEntityPanel && matchedWebFormStep.Visited) {
          prevEntityPanel.attr("data-completed", true);
        }
        prevEntityPanel = $(this);
      });
    };
    
    const _updateWebFormSession = async function (e, webFormStep) {
      const webFormStepId = webFormStep?.Id;
      if (!webFormStepId) {
        return;
      }

      const webFormStepName = webFormStep.Name;

      try {
        FRCC.Portal.Utilities.showProgressIndicator(
          `Navigating to Form Step ${webFormStepName}`
        );

        window.location.reload();
      } catch (error) {
        FRCC.Portal.Utilities.hideProgressIndicator();
        console.error(error);
      }
    };

    const _orderWebFormSteps = function (webFormSteps) {
      const stepMap = {};
      webFormSteps.forEach((step) => {
        stepMap[step.Id] = step;
      });

      // Find the starting step (a step that isn't anyone else's `NextStepId`)
      const nextStepIds = new Set(webFormSteps.map((step) => step.NextStepId));
      const startingStep = webFormSteps.find(
        (step) => !nextStepIds.has(step.Id)
      );

      // Reorder the steps based on the NextStepId
      const orderedSteps = [];
      let currentStep = startingStep;
      let order = 0;

      while (currentStep) {
        orderedSteps.push({ ...currentStep, Order: order });
        order++;

        const nextId = currentStep.NextStepId;
        currentStep = stepMap[nextId] || null;
      }

      return orderedSteps;
    };
    */

    /**
     * Hide the instructions content if there isnt anything written for the instructions text
     */
    var _hideInstructionsIfEmpty = function () {
      const instructionsBlock = document.querySelector(
        '[data-id="instructions-block"]'
      );

      if (instructionsBlock) {
        const instructionsContent = instructionsBlock.querySelector(
          ".override-rich-text"
        );

        if (instructionsContent.innerText === "") {
          instructionsBlock.style.display = "none";
        }
      }
    };

    /**
     * Reverse the order of the boolean yes/no cells so 'Yes' is first,
     * Also deselect the 'No' option if the next step isnt visited yet as
     * a way to determine if those choices were done by the user since they
     * would have chosen them to get to the following step
     */
    var _reverseRadioBooleanCells = function () {
      $("iframe[data-page]").on("load", function (e) {
        try {
          _reverseRadioBooleanCellsHelper(
            e.currentTarget.contentWindow.document
          );
        } catch (error) {
          handleError(error);
        }
      });

      _reverseRadioBooleanCellsHelper(document);
    };

    var _reverseRadioBooleanCellsHelper = function (documentContext) {
      const yesNoCells = documentContext.querySelectorAll(
        ".boolean-radio-cell .boolean-radio"
      );

      if (yesNoCells.length) {
        yesNoCells.forEach((cell) => {
          const disabledRadioSet = cell.querySelectorAll(".aspNetDisabled");
          let noOptionInput;
          let yesOptionInput;
          let yesOptionLabel;

          // If the radios are readonly we need to get the parent spans separating the two options,
          // markup is different between readonly/disabled an enabled
          if (disabledRadioSet.length) {
            noOptionInput = cell
              .querySelectorAll("input")[0]
              .closest("span.aspNetDisabled");
            yesOptionInput = cell
              .querySelectorAll("input")[1]
              .closest(".aspNetDisabled");
            yesOptionLabel = cell
              .querySelectorAll("label")[1]
              .closest(".aspNetDisabled");
          } else {
            noOptionInput = cell.querySelectorAll("input")[0];
            yesOptionInput = cell.querySelectorAll("input")[1];
            yesOptionLabel = cell.querySelectorAll("label")[1];
          }

          cell.insertBefore(yesOptionInput, noOptionInput);
          cell.insertBefore(yesOptionLabel, noOptionInput);
        });
      }
    };

    /**
     * Deselect the 'No' option if the next step isnt visited yet as
     * a way to determine if those choices were done by the user since they
     * would have chosen them to get to the following step
     */
    var _uncheckDefaultedRadioButtons = function () {
      const $insertIFrame = $(".modal-form-insert iframe[data-page]");

      $insertIFrame.on("load", function (e) {
        try {
          _uncheckDefaultedRadioButtonsHelper(
            e.currentTarget.contentWindow.document,
            true
          );
        } catch (error) {
          handleError(error);
        }
      });

      _uncheckDefaultedRadioButtonsHelper(document);
    };

    var _uncheckDefaultedRadioButtonsHelper = function (
      documentContext,
      isIFrameInsertForm = false
    ) {
      const currentStep = documentContext.querySelector(
        ".list-group-item.active"
      );

      const yesNoCells = documentContext.querySelectorAll(
        ".boolean-radio-cell .boolean-radio"
      );

      if (isIFrameInsertForm) {
        if (yesNoCells.length) {
          yesNoCells.forEach((cell) => {
            if (!cell.hasAttribute("disabled")) {
              // input[value="0"] is the 'No' option
              cell.querySelector('input[value="0"]').checked = false;
            }
          });
        }
      }

      if (!!currentStep === true) {
        const isNextStepCompleted =
          currentStep.nextElementSibling?.dataset.visited === "true";

        if (yesNoCells.length) {
          yesNoCells.forEach((cell) => {
            if (!isNextStepCompleted && !cell.hasAttribute("disabled")) {
              // input[value="0"] is the 'No' option
              cell.querySelector('input[value="0"]').checked = false;
            }
          });
        }
      }

      /* For the Download View page, we will instead query for and uncheck all
       * Yes / No fields in any basic forms that have been tagged as not yet visited */
      documentContext
        .querySelectorAll(
          "#EntityFormPanel[data-completed=false] .boolean-radio-cell .boolean-radio"
        )
        .forEach((cell) => {
          cell.querySelector('input[value="0"]').checked = false;
        });
    };

    var _removeAriaLabels = function () {
      const allAriaLabelElements = document.querySelectorAll("[aria-label]");
      allAriaLabelElements.forEach((element) => {
        if (
          !(
            (element.getAttribute("aria-label") === "action menu" &&
              element.tagName.toLowerCase() === "button") ||
            element.classList.contains("multi-step-form-list-group")
          )
        ) {
          element.removeAttribute("aria-label");
        }
      });
    };

    var _removeAriaReadOnlyOnTables = function () {
      const viewGridDivs = document.querySelectorAll(
        "div.view-grid.table-responsive"
      );
      viewGridDivs.forEach((div) => {
        const ariaReadonlyElements = div.querySelectorAll("[aria-readonly]");
        ariaReadonlyElements.forEach((element) => {
          element.removeAttribute("aria-readonly");
        });
      });
    };

    var _applyDatePickerAriaAttributes = function () {
      $("iframe[data-page]").on("load", function (e) {
        try {
          _applyDatePickerAriaAttributesHelper(
            e.currentTarget.contentWindow.document
          );
        } catch (error) {
          handleError(error);
        }
      });

      _applyDatePickerAriaAttributesHelper(document);
    };

    var _applyDatePickerAriaAttributesHelper = function (documentContext) {
      const datePickerEls =
        documentContext.getElementsByClassName("datetimepicker");

      if (datePickerEls.length) {
        datePickerEls.forEach((datepicker) => {
          const datePickerInput = datepicker.querySelector("input");
          const datePickerButton = datepicker.querySelector(
            'span[role="button"]'
          );

          datePickerButton.setAttribute(
            "aria-controls",
            `${datePickerInput.id}-datetimepicker`
          );

          // Mutation observer for date widget showing
          const config = {
            attributes: true,
            attributeFilter: ["aria-expanded"]
          };

          const callback = function (mutationsList) {
            for (const mutation of mutationsList) {
              if (
                mutation.type === "attributes" &&
                mutation.attributeName === "aria-expanded"
              ) {
                const isAriaExpanded =
                  datePickerButton.getAttribute("aria-expanded") === "true";
                if (isAriaExpanded) {
                  datepicker
                    .querySelector(".bootstrap-datetimepicker-widget")
                    .setAttribute("id", `${datePickerInput.id}-datetimepicker`);
                }
              }
            }
          };

          const observer = new MutationObserver(callback);
          observer.observe(datePickerButton, config);
        });
      }
    };

    var _addTableCaptions = function () {
      $("iframe[data-page]").on("load", function (e) {
        try {
          _addTableCaptionsHelper(e.currentTarget.contentWindow.document);
        } catch (error) {
          handleError(error);
        }
      });

      _addTableCaptionsHelper(document);
    };

    var _addTableCaptionsHelper = function (documentContext) {
      const tableSubgrids =
        documentContext.querySelectorAll(".cell.subgrid-cell");

      tableSubgrids.forEach((tableSubgrid) => {
        const tableEl = tableSubgrid.querySelector("table");
        const tableLabel = tableSubgrid.querySelector("h3 .field-label");
        const captionEl = document.createElement("caption");

        captionEl.classList.add("sr-only");
        captionEl.append(tableLabel ? tableLabel.innerText : "Table");

        tableEl.prepend(captionEl);
      });
    };

    var _addDocumentFileAriaLabels = function (mutationsList) {
      for (const mutation of mutationsList) {
        if (mutation.type === "childList") {
          mutation.addedNodes.forEach((node) => {
            if (node.nodeType === Node.ELEMENT_NODE && node.tagName === "TR") {
              const fileButton = node.querySelector("td:first-child button");
              const fileName = fileButton?.innerText;

              fileButton.setAttribute(
                "aria-label",
                `${fileName}, download document`
              );
            }
          });
        }
      }
    };

    var _addDocumentDownloadAccessibleText = function () {
      const documentFilesTable = document.getElementById("file-upload-table");

      if (documentFilesTable) {
        const observer = new MutationObserver(_addDocumentFileAriaLabels);

        const config = { childList: true, subtree: true };
        observer.observe(documentFilesTable, config);
      }
    };

    var _fixTableActionsHeader = function () {
      $("iframe[data-page]").on("load", function (e) {
        try {
          _fixTableActionsHeaderHelper(e.currentTarget.contentWindow.document);
        } catch (error) {
          handleError(error);
        }
      });

      _fixTableActionsHeaderHelper(document);
    };

    var _fixTableActionsHeaderHelper = function (documentContext) {
      // Find TH and remove sr-only class
      const tableActionsTh = documentContext.querySelectorAll(
        `th[data-th="<span class='sr-only'>Actions</span>"]`
      );

      tableActionsTh.forEach((thElement) => {
        const srElement = thElement.querySelector(".sr-only");
        srElement?.classList.remove("sr-only");
      });
    };

    /**
     * The table class in application forms is preventing the table from being usable for mobile
     * Adjust parent table class names and get rid of striped table class
     *
     */
    var _updateTableResponsive = function () {
      const tableResponsiveElements =
        document.querySelectorAll(`.table-responsive`);

      tableResponsiveElements.forEach((tableResponsive) => {
        const table = tableResponsive.querySelector("table");

        tableResponsive.classList.remove("table-responsive");
        tableResponsive.classList.add("table-scroll-wrapper");

        table.className =
          "table table-custom table-mobile table-fluid-label-width-md";
      });
    };

    /**
     * The Action buttons in the application tables use the wrong icon and need
     * extra classes to help display it correctly on mobile
     * Use this utility function to take the dropdown button and create the new icon and sr-only text
     * and also apply parent td classes to help with mobile layout
     *
     */
    var _setUpActionButton = function (button) {
      const dropdownTd = button.closest("td");
      const dropdownButtonIcon = button.querySelector(".fa");
      const mobileActionsElement = document.createElement("span");
      const mobileActionsCaret = document.createElement("span");

      mobileActionsElement.className = "visible-xs";
      mobileActionsElement.innerText = "Actions";

      mobileActionsCaret.setAttribute("aria-hidden", "true");
      mobileActionsCaret.className = "caret ml-2";

      mobileActionsElement.append(mobileActionsCaret);
      button.append(mobileActionsElement);

      dropdownTd.classList.add("contains-component", "text-right");

      button.className =
        "btn btn-default btn-xs-breakpoint-md btn-xs-breakpoint-lg";

      if (dropdownButtonIcon) {
        dropdownButtonIcon.className =
          "glyphicon glyphicon-option-horizontal hidden-xs";
      }
    };

    /**
     * Update the tables action buttons when they intitially load, when new elements are added via create or delete
     * the icons were reverting back to incorrect icon, so there is a mutation observer to listen for changes in the table
     * in the `_updateTableActionIcons` function
     *
     */
    var _updateInitialTableActions = function () {
      // Find action buttons and update classes and icon
      const tableActionsDropdown = document.querySelectorAll(`td .dropdown`);

      tableActionsDropdown.forEach((dropdown) => {
        const dropdownButton = dropdown.querySelector("button");
        _setUpActionButton(dropdownButton);
      });
    };

    /**
     *
     * Mutation observer function for changes in application table
     */
    var _updateNewTableActions = function (mutationsList) {
      for (const mutation of mutationsList) {
        if (mutation.type === "childList") {
          mutation.addedNodes.forEach((node) => {
            if (node.nodeType === Node.ELEMENT_NODE) {
              const dropdownButton = node.querySelectorAll(".dropdown button");
              dropdownButton.forEach((button) => {
                _setUpActionButton(button);
              });
            }
          });
        }
      }
    };

    /**
     * Listen to application table changes to update action button icons
     */
    var _updateTableActionIcons = function () {
      const tableGrids = document.querySelectorAll(".view-grid table");
      if (tableGrids.length) {
        // Update action buttons on load
        _updateInitialTableActions();

        // Update action buttons when they are added to table as new rows
        const observer = new MutationObserver(_updateNewTableActions);
        const config = { childList: true, subtree: true };

        tableGrids.forEach((table) => {
          observer.observe(table, config);
        });
      }
    };

    return {
      loadGlobalAccessibilityModifications:
        loadGlobalAccessibilityModifications,
      loadApplicationAccessibilityModifications:
        loadApplicationAccessibilityModifications,
      resetPageTabbing: resetPageTabbing
    };
  })();
})(this);
